package org.xbib.interlibrary.imap;

import javax.mail.Flags;
import javax.mail.Folder;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Session;
import javax.mail.Store;
import javax.mail.search.FlagTerm;
import javax.mail.search.SearchTerm;
import java.net.URI;
import java.util.Objects;
import java.util.Properties;
import java.util.function.Consumer;

/**
 * A wrapper class for IMAP functionality.
 */
public class ImapClient {

    private final Store store;

    private ImapClient(String url, String username, String password) throws MessagingException {
        Properties properties = createEnvironment(Objects.requireNonNull(url));
        Session session = Session.getDefaultInstance(properties, null);
        this.store = session.getStore("imap");
        this.store.connect(username, password);
    }

    public static ImapClient newInstance(String url) throws MessagingException {
        return new ImapClient(url, null, null);
    }

    public static ImapClient newInstance(String url, String username, String password) throws MessagingException {
        return new ImapClient(url, username, password);
    }

    public void close() throws MessagingException {
        if (store != null) {
            store.close();
        }
    }

    public Boolean exist(String folderName) throws MessagingException {
        Folder folder = store.getFolder(folderName);
        return folder.exists();
    }

    public void eachFolder(String folderName,
                           String folderPattern,
                           Consumer<Folder> consumer) throws MessagingException {
        Folder folder = store.getFolder(folderName);
        if (folder.exists()) {
            folder.open(Folder.READ_ONLY);
            Folder[] folders = folder.list(folderPattern);
            for (Folder f : folders) {
                consumer.accept(f);
            }
            folder.close(false);
        }
    }

    public void create(String folderName) throws MessagingException {
        Folder folder = store.getFolder(folderName);
        if (!folder.exists()) {
            folder.create(Folder.HOLDS_MESSAGES | Folder.READ_WRITE);
        }
        folder.close(false);
    }

    public void delete(String folderName) throws MessagingException {
        Folder folder = store.getFolder(folderName);
        if (folder.exists()) {
            folder.delete(true);
        }
        folder.close(true);
    }

    public void expunge(String folderName) throws MessagingException {
        Folder folder = store.getFolder(folderName);
        if (folder.exists()) {
            folder.expunge();
        }
        folder.close(false);
    }

    public Integer messageCount(String folderName) throws MessagingException {
        Folder folder = store.getFolder(folderName);
        if (folder.exists()) {
            return folder.getMessageCount();
        }
        return -1;
    }

    public void eachMessage(String folderName, Consumer<Message> consumer) throws MessagingException {
        Folder folder = store.getFolder(folderName);
        if (folder.exists()) {
            folder.open(Folder.READ_ONLY);
            Message[] messages = folder.getMessages();
            for (Message message : messages) {
                consumer.accept(message);
            }
            folder.close(false);
        }
    }

    public void eachMessage(String folderName, int start, int end, Consumer<Message> consumer) throws MessagingException {
        Folder folder = store.getFolder(folderName);
        if (folder.exists()) {
            folder.open(Folder.READ_ONLY);
            Message[] messages = folder.getMessages(start, end);
            for (Message message : messages) {
                consumer.accept(message);
            }
            folder.close(false);
        }
    }

    public void eachSearchedMessage(String folderName, Flags flags, Consumer<Message> consumer) throws MessagingException {
        Folder folder = store.getFolder(folderName);
        if (folder.exists()) {
            folder.open(Folder.READ_ONLY);
            FlagTerm flagTerm = new FlagTerm(flags, false);
            Message[] messages = folder.search(flagTerm);
            for (Message message : messages) {
                consumer.accept(message);
            }
            folder.close(false);
        }
    }

    public void eachSearchedMessage(String folderName, SearchTerm searchTerm, Consumer<Message> consumer) throws MessagingException {
        Folder folder = store.getFolder(folderName);
        if (folder.exists()) {
            folder.open(Folder.READ_ONLY);
            Message[] messages = folder.search(searchTerm);
            for (Message message : messages) {
                consumer.accept(message);
            }
            folder.close(false);
        }
    }

    private static Properties createEnvironment(String urlSpec) {
        URI uri = URI.create(urlSpec);
        Properties env = new Properties();
        env.setProperty("mail.store.protocol", "imap");
        env.setProperty("mail.imap.host", uri.getHost());
        env.setProperty("mail.imap.port", Integer.toString(uri.getPort()));
        boolean secure = uri.getScheme().equals("imaps") || 993 == uri.getPort();
        env.setProperty("mail.imap.ssl.enable", secure ? "true" : "false");
        return env;
    }
}
