package org.xbib.interlibrary.smtp;

import javax.activation.DataHandler;
import javax.mail.Address;
import javax.mail.Authenticator;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.PasswordAuthentication;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.AddressException;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;
import javax.mail.util.ByteArrayDataSource;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URI;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Properties;

public class SmtpClient {

    private static final URI DEFAULT = URI.create("smtp://localhost:25/");

    private final URI uri;

    private final String username;

    private final String password;

    private boolean debug;

    private SmtpClient(URI uri, String username, String password) {
        this.uri = uri;
        this.username = username;
        this.password = password;
    }

    public static SmtpClient newInstance() {
        return new SmtpClient(DEFAULT, null, null);
    }

    public static SmtpClient newInstance(URI uri) {
        return new SmtpClient(uri, null, null);
    }

    public static SmtpClient newInstance(URI uri, String username, String password) {
        return new SmtpClient(uri, username, password);
    }

    public URI getURI() {
        return uri;
    }

    public void setDebug(boolean debug) {
        this.debug = debug;
    }

    public void sendMail(Map<String, Object> map) throws MessagingException, UnsupportedEncodingException {
        String sender = (String) map.get("sender");
        String from = (String) map.get("from");
        String personal = (String) map.get("personal");
        String allRecipients = (String) map.get("to");
        String allCC = (String) map.get("cc");
        String allBCC = (String) map.get("bcc");
        String subject = (String) map.get("subject");
        String body = (String) map.get("message");
        sendMail(sender, from, personal, allRecipients, allCC, allBCC, subject, body);
    }

    public void sendMail(String sender,
                         String from,
                         String personal,
                         String allRecipients,
                         String allCC,
                         String allBCC,
                         String subject,
                         String body) throws MessagingException, UnsupportedEncodingException {
        Properties properties = createProperties();
        Session session = username != null ?
                Session.getDefaultInstance(properties, new SMTPAuthenticator()) :
                Session.getDefaultInstance(properties);
        if (debug) {
            session.setDebug(debug);
            session.setDebugOut(System.err);
        }
        MimeMessage message = new MimeMessage(session);
        message.setSentDate(new Date());
        if (sender != null) {
            message.setSender(new InternetAddress(sender));
        }
        message.setFrom(new InternetAddress(from, personal));
        if (allRecipients != null ) {
            for (String recipient : allRecipients.split(",")) {
                message.addRecipient(Message.RecipientType.TO, new InternetAddress(recipient.trim()));
            }
        }
        if (allCC != null) {
            for (String cc : allCC.split(",")) {
                if (cc != null && !cc.equals("-")) {
                    message.addRecipient(Message.RecipientType.CC, new InternetAddress(cc.trim()));
                }
            }
        }
        if (allBCC != null) {
            for (String bcc : allBCC.split(",")) {
                if (bcc != null && !bcc.equals("-")) {
                    message.addRecipient(Message.RecipientType.BCC, new InternetAddress(bcc.trim()));
                }
            }
        }
        message.setSubject(subject);
        Multipart multipart = new MimeMultipart("mixed");
        multipart.addBodyPart(createTextPart(body));
        message.setContent(multipart);
        Transport.send(message);
    }

    public List<Address> createStrictAddress(List<String> list) throws AddressException {
        List<Address> result = new ArrayList<>();
        for (String string : list) {
            Address address = createStrictAddress(string);
            if (address != null) {
                result.add(address);
            }
        }
        return result;
    }

    public Address createStrictAddress(String string) throws AddressException {
        return string != null && !string.isEmpty() ? new InternetAddress(string, true) : null;
    }

    public Address createAddress(String string) throws AddressException {
        if (string == null || string.isEmpty()) {
            return null;
        }
        InternetAddress internetAddress = new InternetAddress(string, true);
        internetAddress.validate();
        return internetAddress;
    }

    public Address createAddress(String address, String personal) throws UnsupportedEncodingException, AddressException {
        if (address == null || address.isEmpty()) {
            return null;
        }
        InternetAddress internetAddress = new InternetAddress(address, personal);
        internetAddress.validate();
        return internetAddress;
    }

    public MimeBodyPart createTextPart(String text) throws MessagingException {
        MimeBodyPart mimeBodyPart = new MimeBodyPart();
        mimeBodyPart.setHeader("Content-Transfer-Encoding", "base64");
        mimeBodyPart.setText(text, "utf-8", "plain");
        return mimeBodyPart;
    }

    public MimeBodyPart createTextPart(String disposition,
                                       String fileName,
                                       String text) throws MessagingException {
        MimeBodyPart mimeBodyPart = new MimeBodyPart();
        mimeBodyPart.setHeader("Content-Transfer-Encoding", "base64");
        mimeBodyPart.setText(text, "utf-8", "plain");
        mimeBodyPart.setDisposition(disposition);
        mimeBodyPart.setFileName(fileName);
        return mimeBodyPart;
    }
    public MimeBodyPart createBinaryPart(String contentType,
                                         String disposition,
                                         String fileName,
                                         byte[] bytes) throws MessagingException, IOException {
        MimeBodyPart mimeBodyPart = new MimeBodyPart();
        mimeBodyPart.setHeader("Content-Transfer-Encoding", "base64");
        mimeBodyPart.setDisposition(disposition);
        ByteArrayInputStream inputStream = new ByteArrayInputStream(bytes);
        ByteArrayDataSource byteArrayDataSource = new ByteArrayDataSource(inputStream, contentType);
        mimeBodyPart.setDataHandler(new DataHandler(byteArrayDataSource));
        mimeBodyPart.setFileName(fileName);
        return mimeBodyPart;
    }

    public boolean send(String subject,
                        Address from,
                        Address replyTo,
                        Address to,
                        Address cc,
                        List<Address> bcc,
                        List<MimeBodyPart> mimeBodyparts) throws MessagingException {
        Properties properties = createProperties();
        Session session = username != null ?
                Session.getDefaultInstance(properties, new SMTPAuthenticator()) :
                Session.getDefaultInstance(properties);
        MimeMessage message = new MimeMessage(session);
        message.setSentDate(new Date());
        if (subject == null) {
            return false;
        }
        message.setSubject(subject);
        if (from == null) {
            return false;
        }
        message.setFrom(from);
        if (replyTo != null) {
            message.setReplyTo(new Address[] { replyTo } );
        }
        if (to == null) {
            return false;
        }
        message.setRecipients(Message.RecipientType.TO, new Address[] { to } );
        if (cc != null) {
            message.setRecipients(Message.RecipientType.CC, new Address[] { cc } );
        }
        if (bcc != null && !bcc.isEmpty()) {
            message.setRecipients(Message.RecipientType.BCC, bcc.toArray(new Address[0]));
        }
        if (mimeBodyparts != null && !mimeBodyparts.isEmpty()) {
            Multipart multipart = new MimeMultipart("related");
            for (MimeBodyPart mimeBodyPart : mimeBodyparts) {
                multipart.addBodyPart(mimeBodyPart);
            }
            message.setContent(multipart);
        }
        Transport.send(message);
        return true;
    }

    public boolean send(String subject,
                        Address from,
                        List<Address> replyTo,
                        List<Address> to,
                        List<Address> cc,
                        List<Address> bcc,
                        List<MimeBodyPart> mimeBodyparts) throws MessagingException {
        Properties properties = createProperties();
        Session session = username != null ?
                Session.getDefaultInstance(properties, new SMTPAuthenticator()) :
                Session.getDefaultInstance(properties);
        MimeMessage message = new MimeMessage(session);
        message.setSentDate(new Date());
        if (subject == null) {
            return false;
        }
        message.setSubject(subject);
        if (from == null) {
            return false;
        }
        message.setFrom(from);
        if (replyTo != null && !replyTo.isEmpty()) {
            message.setReplyTo(replyTo.toArray(new Address[0]));
        }
        if (to == null || to.isEmpty()) {
            return false;
        }
        message.setRecipients(Message.RecipientType.TO, to.toArray(new Address[0]));
        if (cc != null && !cc.isEmpty()) {
            message.setRecipients(Message.RecipientType.CC, cc.toArray(new Address[0]));
        }
        if (bcc != null && !bcc.isEmpty()) {
            message.setRecipients(Message.RecipientType.BCC, bcc.toArray(new Address[0]));
        }
        if (mimeBodyparts != null && !mimeBodyparts.isEmpty()) {
            Multipart multipart = new MimeMultipart("related");
            for (MimeBodyPart mimeBodyPart : mimeBodyparts) {
                multipart.addBodyPart(mimeBodyPart);
            }
            message.setContent(multipart);
        }
        Transport.send(message);
        return true;
    }

    private Properties createProperties() {
        Properties properties = new Properties();
        properties.setProperty("mail.smtp.host", uri.getHost());
        properties.setProperty("mail.smtp.port", Integer.toString(uri.getPort()));
        properties.setProperty("mail.smtp.auth",  "false");
        properties.setProperty("mail.smtp.ssl.enable", uri.getScheme().equals("smtps") || 995 == uri.getPort() ? "true" : "false");
        properties.setProperty("mail.debug", String.valueOf(debug));
        return properties;
    }

    private class SMTPAuthenticator extends Authenticator {

        public SMTPAuthenticator() {
        }

        @Override
        public PasswordAuthentication getPasswordAuthentication() {
            return new PasswordAuthentication(username, password);
        }
    }
}
